'use strict';
// *******************************************************
// Respack
// -------------------------------------------------------
// The inner workings of the respack module
// -------------------------------------------------------

// *******************************************
// Module Imports
// -------------------------------------------

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.postRequestParams = exports.getRequestParams = undefined;

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// --------------------------------

// *******************************************
// Implementation
// -------------------------------------------
var OKAY = 'OKAY';
var FAILED = 'FAILED';
var ERROR = 'ERROR';

var GET = function GET(url, headers) {
    return (0, _axios2.default)(url, getRequestParams(headers)).then(handleResponse).catch(handleError);
};

var POST = function POST(url, body, headers) {
    return (0, _axios2.default)(url, postRequestParams(body, headers)).then(handleResponse).catch(handleError);
};

var sendOkay = function sendOkay(body) {
    return {
        status: OKAY,
        body: body || {}
    };
};

var sendFailed = function sendFailed(body) {
    return {
        status: FAILED,
        body: body
    };
};

var sendError = function sendError(body) {
    return {
        status: ERROR,
        body: body
    };
};

var sendOkayRes = function sendOkayRes(res) {
    var body = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var status = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 200;

    return res.status(status).json(sendOkay(body));
};

var sendFailedRes = function sendFailedRes(res) {
    var body = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var status = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 400;

    return res.status(status).json(sendFailed(body));
};

var sendErrorRes = function sendErrorRes(res) {
    var body = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var status = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 400;

    return res.status(status).json(sendError(body));
};

var handleResponse = function handleResponse(res) {
    switch (res.data.status) {
        case OKAY:
            if (res.status === 200) {
                return sendOkay(res.data.body);
            } else {
                return { status: ERROR, error: "Server Error..." };
            }
        case FAILED:
            return sendFailed(res.data.body);
        case ERROR:
            return sendError(res.data.body);
        default:
            return sendError(res.data.body);
    }
};

var handleError = function handleError(err) {
    if (err) {
        return err;
    } else {
        return undefined;
    }
};

var safeAsync = function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(promise, req, res) {
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        return _context.abrupt('return', promise.then(function (result) {
                            switch (result.status) {
                                case OKAY:
                                    return result.body;
                                case FAILED:
                                    sendFailedRes(res, result.body);
                                    return null;
                                case ERROR:
                                    sendErrorRes(res, result.body);
                                    return null;
                                default:
                                    console.warn("We fell through all the RESPACK checks.");
                                    console.warn("Something is DANGEROUSLY wrong!!!!");
                                    return null;
                            }
                        }).catch(function (err) {
                            console.log('ERROR: MAKING API REQUEST');
                            console.log(err);
                            sendErrorRes(res, err.message);
                            return null;
                        }));

                    case 1:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function safeAsync(_x7, _x8, _x9) {
        return _ref.apply(this, arguments);
    };
}();

/**
 * @function getRequestParams
 * Generates a standard get request parameters for a fetch request
 * @params {object} headers - Key Value pairs of request headers
 * @returns {object} params - Basic params for a get request using fetch
 */
var getRequestParams = exports.getRequestParams = function getRequestParams() {
    var headers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { "Content-Type": "application/json" };

    var params = {
        method: "GET",
        headers: headers,
        withCredentials: true,
        data: undefined
    };
    return params;
};

/**
 * @function postRequestParams
 * Generates a standard post request parameters for a fetch request
 * @params {object} body - JSON object to be passed onto the fetch API
 * @params {object} headers - Key Value pairs of request headers
 * @returns {object} params - Basic params for a post request using fetch
 */
var postRequestParams = exports.postRequestParams = function postRequestParams(body) {
    var headers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { "Content-Type": "application/json" };

    var params = {
        method: "POST",
        headers: headers,
        withCredentials: true,
        data: body
    };
    return params;
};

// --------------------------------

// *******************************************
// Module Exports
// -------------------------------------------
var respack = {
    STATUS: {
        OKAY: OKAY,
        FAILED: FAILED,
        ERROR: ERROR
    },
    TYPE: {
        GET: "GET",
        POST: "POST"
    },
    GET: GET,
    POST: POST,
    ASYNC: safeAsync,
    OKAY: sendOkay,
    ERROR: sendError,
    FAILED: sendFailed
};
exports.default = respack;
// --------------------------------